@@ -20,7 +20,9 @@ setup(
     Extension("errors",       ["errors.pyx"]),
     Extension("cpdef",       ["cpdef.pyx"]),
     Extension("range",       ["range.pyx"]),
+    Extension("early_temps",       ["early_temps.pyx"]),
     Extension("ints",       ["ints.pyx"]),
+    Extension("clear",       ["clear.pyx"]),
     Extension("detect_override",       ["detect_override.pyx"]),
     ],
   cmdclass = {'build_ext': build_ext},
